/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/tadt/comparison.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef tadt_comparison
#define tadt_comparison

/*  
*  Routine:  tadt_fs_eq_fs
*
*  Description: This routine compares two fixed span values.  It returns
*		the boolean value TRUE if they are equal and FALSE otherwise.
*
*  Arguments: 	span1, span2 -- (IN) : spans to be compared
*
*  Return Value: TRUE if span1 == span2, FALSE otherwise
*  
*  Errors: None
*  
*  Side Effects: None
*/

#ifndef INDETERMINATE
bool tadt_fs_eq_fs(span_type span1, span_type span2);
#else
#define tadt_fs_eq_fs(span1, span2) tadti_e_eq_e(100, span1, span2);
#endif

/*  
*  Routine:  tadt_fs_lt_fs
*
*  Description: This routine compares two fixed span values.  It returns
*		the boolean value TRUE if the first span is less than
*		the second span and FALSE otherwise.
*
*  Arguments: 	span1, span2 -- (IN) : spans to be compared
*
*  Return Value: TRUE if span1 < span2, FALSE otherwise
*  
*  Errors: None
*  
*  Side Effects: None
*/

#ifndef INDETERMINATE
bool tadt_fs_lt_fs(span_type span1, span_type span2);
#else
#define tadt_fs_lt_fs(span1, span2) tadti_fs_lt_fs(100, span1, span2)
#endif

/*  
*  Routine:  tadt_fs_gt_fs
*
*  Description: This routine compares two fixed span values.  It returns
*		the boolean value TRUE if the first span is greater than
*		the second span and FALSE otherwise.
*
*  Arguments: 	span1, span2 -- (IN) : spans to be compared
*
*  Return Value: TRUE if span1 > span2, FALSE if span1 =< span2
*  
*  Errors: None
*
*  Side Effects: None
*/

#ifndef INDETERMINATE
bool tadt_fs_gt_fs(span_type span1, span_type span2);
#else
#define tadt_fs_gt_fs(span1, span2) tadti_fs_gt_fs(100, span1, span2)
#endif

/*  
*  Routine:  tadt_e_eq_e
*
*  Description: This routine compares two event values.  It returns
*		the boolean value TRUE if the events are equal and
*		FALSE otherwise.  Two events are defined to be
*		equal if they occur during the same chronon,
*		not if they occur at the same time.     
*		The size of a chronon is resolution dependent.
*
*  Arguments: 	event1, event2 -- (IN) : events to be compared
*
*  Return Value: TRUE if event1 = event2, FALSE if event1 <> event2
*  
*  Errors: None
*  
*  Side Effects: None
*/

#ifndef INDETERMINATE
bool tadt_e_eq_e(event_type event1, event_type event2);
#else
#define tadt_e_eq_e(ev1, ev2) tadti_e_eq_e(100, ev1, ev2)
#endif

/*  
*  Routine:  tadt_e_precedes_e
*
*  Description: This routine compares two event values.  It returns
*		the boolean value TRUE if the first event precedes the
*		second event and FALSE otherwise.
*
*  Arguments: 	event1, event2 -- (IN) : events to be compared
*
*  Return Value: TRUE if event1 < event2, FALSE if event1 >= event2
*  
*  Errors: None
*  
*  Side Effects: None
*/

#ifndef INDETERMINATE
bool tadt_e_precedes_e(event_type event1, event_type event2);
#else
#define tadt_e_precedes_e(ev1, ev2) tadti_e_precedes_e(100, ev1, ev2)
#endif

/*  
*  Routine:  tadt_e_precedes_i
*
*  Description: This routine compares an event and an interval.  It returns
*		the boolean value TRUE if the event precedes the
*		beginning of the interval and FALSE otherwise.
*
*  Arguments: 	event, interval -- (IN) : event and interval to be compared
*
*  Return Value: TRUE if event < begin(interval),
*                FALSE if event >= begin(interval)
*  
*  Errors: None
*  
*  Side Effects: None
*/

#ifndef INDETERMINATE
bool tadt_e_precedes_i(event_type event, interval_type interval);
#else
#define tadt_e_precedes_i(event, interval) tadti_e_precedes_i(100, event, interval)
#endif

/*  
*  Routine:  tadt_e_overlaps_i
*
*  Description: This routine compares an event and an interval.  It returns
*		the boolean value TRUE if the event overlaps the
*		interval and FALSE otherwise.
*
*  Arguments: 	event, interval -- (IN) : event and interval to be compared
*
*  Return Value: TRUE if begin(interval) <= event <= end(interval)
*		 FALSE if event < begin(interval) or event > end(interval)
*  
*  Errors: None
*  
*  Side Effects: None
*/

#ifndef INDETERMINATE
bool tadt_e_overlaps_i(event_type event, interval_type interval);
#else
#define tadt_e_overlaps_i(event, interval) tadti_e_overlaps_i(100, event, interval)
#endif

/*  
*  Routine:  tadt_i_precedes_e
*
*  Description: This routine compares an interval and an event.  It returns
*		the boolean value TRUE if the interval precedes the
*		event and FALSE otherwise.
*
*  Arguments: 	interval, event -- (IN) : interval and event to be compared
*
*  Return Value: TRUE if end(interval) < event
*		 FALSE if event <= end(interval)
*  
*  Errors: None
*
*  Side Effects: None
*/

#ifndef INDETERMINATE
bool tadt_i_precedes_e(interval_type interval, event_type event);
#else
#define tadt_i_precedes_e(interval, event) tadti_i_precedes_e(100, interval, event)
#endif

/*  
*  Routine:  tadt_i_precedes_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the end of the first interval 
*		precedes the beginning of the second.
*
*  Arguments: 	interval1, interval2 -- (IN) : intervals to be compared
*
*  Return Value: TRUE if end(interval1) < begin(interval2)
*		 FALSE if end(interval1) >= begin(interval2)
*  
*  Errors: None
*  
*  Side Effects: None
*/

#ifndef INDETERMINATE
bool tadt_i_precedes_i(interval_type interval1, interval_type interval2);
#else
#define tadt_i_precedes_i(interval1, interval2) tadti_i_precedes_i(100, interval1, interval2)
#endif

/*  
*  Routine:  tadt_i_eq_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the two intervals are
*		equal and FALSE otherwise.
*
*  Arguments: 	interval1, interval2 -- (IN) : intervals to be compared
*
*  Return Value: bool TRUE if interval1 = interval2
*		 bool FALSE if interval1 <> interval2
*  
*  Errors: None
*  
*  Side Effects: None
*/

#ifndef INDETERMINATE
bool tadt_i_eq_i(interval_type interval1, interval_type interval2);
#else
#define tadt_i_eq_i(interval1, interval2) tadti_i_eq_i(100, interval1, interval2)
#endif

/*  
*  Routine:  tadt_i_meets_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the end of the first interval
*		is the beginning of the second and FALSE otherwise.
*
*  Arguments: 	interval1, interval2 -- (IN) : intervals to be compared
*
*  Return Value: TRUE if end(interval1) = begin(interval2)
*                FALSE if end(interval1) <> begin(interval2)
*  
*  Errors: None
*  
*  Side Effects: None
*/

#ifndef INDETERMINATE
bool tadt_i_meets_i(interval_type interval1, interval_type interval2);
#else
#define tadt_i_meets_i(interval1, interval2) tadti_i_meets_i(100, interval1, interval2)
#endif

/*  
*  Routine:  tadt_i_overlaps_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the intervals share at least
*		one chronon in common and FALSE otherwise.
*
*  Arguments: 	interval1, interval2 -- (IN) : intervals to be compared
*
*  Return Value: TRUE if interval1 overlaps interval2
*                FALSE if interval1 doesn't overlap interval2
*  
*  Errors: None
*  
*  Side Effects: None
*/

#ifndef INDETERMINATE
bool tadt_i_overlaps_i(interval_type interval1, interval_type interval2);
#else
#define tadt_i_overlaps_i(interval1, interval2) tadti_i_overlaps_i(100, interval1, interval2)
#endif

/*  
*  Routine:  tadt_i_contains_i
*
*  Description: This routine compares two intervals.  It returns
*		the boolean value TRUE if the first interval contains
*		the second interval and FALSE otherwise.
*
*  Arguments: 	interval1, interval2 -- (IN) : intervals to be compared
*
*  Return Value: TRUE if interval1 contains interval2
*                FALSE if interval1 doesn't contain interval2
*  
*  Errors: None
*  
*  Side Effects: None
*/

#ifndef INDETERMINATE
bool tadt_i_contains_i(interval_type interval1, interval_type interval2);
#else
#define tadt_i_contains_i(i1, i2) tadti_i_contains_i(100, i1, i2)
#endif

#endif
