/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/tadt/ext.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef tadt_ext
#define tadt_ext

typedef enum {NEG, POS} tadt_sign_type;

typedef enum {
   tadt_ok,
   tadt_invalid_size, /*time-stamp space error*/
   tadt_invalid_interval, /*interval error*/
   tadt_truncation,       /*had to truncate time-stamp*/
   tadt_bad_negation, /*tried to negate a nonnegatable value*/
   tadt_undefined /*tried something with an undefined value*/
} tadt_error_type;

typedef struct {
  int scale;
  int range;
  int precision;
} precision_space_type;
typedef precision_space_type *precision_type;

typedef enum { packed_determinate, 
               packed_special, 
	       packed_nowrelative,

#ifdef INDETERMINATE
               packed_indeterminate,
               packed_indeterminate_nowrelative,
               packed_nowrelative_indeterminate,
               packed_chunked, 
               packed_chunked_nonstandard,
	       packed_general,
               packed_general_nonstandard,
#endif
               PACKED_MAX_RESOLUTIONS_OF_EVENTS,

} packed_resolution_type;

typedef enum {
               upack,
	       special,
	       nowrelative,
	       upackind,
	       indistinguishable,
	       incomparable,
/* From Here on out, these types aren't actual timestamp types */
               upack32,upack32upack32,upack32upack64,upack32upack96,
                       upack32upackind,upack32special,specialupack32,
		       upack32indistinguishable, upack32incomparable,
               upack64,upack64upack32,upack64upack64,upack64upack96,
                       upack64upackind,upack64special,specialupack64,
		       upack64indistinguishable, upack64incomparable,
               upack96,upack96upack32,upack96upack64,upack96upack96,
                       upack96upackind,upack96special,specialupack96,
		       upack96indistinguishable, upack96incomparable,
               upackind2,upackindupack32,upackindupack64,upackindupack96,
                       upackindupackind,upackindspecial,specialupackind,
		       upackindindistinguishable, upackindincomparable,
               indistinguishable2,indistinguishableupack32,
		       indistinguishableupack64,indistinguishableupack96,
                       indistinguishableupackind,indistinguishablespecial,
		       specialindistinguishable, 
		       indistinguishableindistinguishable, 
		       indistinguishableincomparable,
               incomparable2,incomparableupack32,
		       incomparableupack64,incomparableupack96,
                       incomparableupackind,incomparablespecial,
		       specialincomparable, 
		       incomparableindistinguishable, 
		       incomparableincomparable,
               specialspecial,
               MAX_RESOLUTIONS_OF_EVENTS,
} resolution_type;


/* Ordering is important for tadt routines */
typedef enum { beginning, 
               forever, 
               negative_all_of_time, 
               all_of_time, 
	       /* initiation,     Not implemented */
	       /* until_changed,  Not implemented */
               undefined 
} special_value_type;

/*
* The internal "space" datatypes are defined elsewhere
*/
#include "../tadt_internal/int.h"

typedef event_space_type *event_type;

typedef span_space_type *span_type;

typedef interval_space_type *interval_type;

typedef timestamp_space_type *timestamp_type;

/* 
*  Unpacked time information structure
*/

typedef struct {
  int32 seconds32;
  int64 seconds64; 
  int96 seconds96;      
  resolution_type resolution;  
  special_value_type special;
  size_type size;
} seconds_space_type;

typedef seconds_space_type *seconds_type;

/* 
*  Unpacked variable span information structure
*/

typedef union {
  unsigned int               word[sizeof(vspan_type)/4];
  unsigned short             sword[sizeof(vspan_type)/2];
  unsigned char              byte[sizeof(vspan_type)];
} vs_data_type;

typedef struct {
  unsigned int               cal_id; /* only values < 32 allowed */
  vs_data_type               data; 
} vs_space_type;

typedef vs_space_type *vs_type;

typedef struct {
  precision_space_type precision;
  resolution_type resolution;
} context_space_type;

typedef context_space_type *context_type;

/* 
* Aggregate structures
*/

typedef span_type sum_state_fs;

typedef struct {
  int  count;
  event_type sum_of_events;
} *avg_state_e;

typedef struct {
  int count;
  span_type sum_of_spans;
} *avg_state_fs;

#endif
