/*
 * 64 bit routines
 *  Nick Kline, Oct 3, 1993
 * 
 * for the following d is the destination, a,b are the operands
 */

#ifdef __GNUC__
#define _real_64bit_ints_
#endif

#ifdef _real_64bit_ints_

/* 32 bit support */
typedef long int32;
typedef unsigned long uint32;

/* 64 bit support */
typedef long long int64;
typedef unsigned long long uint64;

/* 128 bit support */
typedef struct {
	int64 high;
	int64 low;
    } int128;

/* 96 bit support */
typedef int32 int96;

/* constants as in an array constant */
#define  int64_xconst(high, low) 0x ## high ## low ## ll
#define  int64_dconst(high, low)       high ## low ## ll
#define uint64_xconst(high, low) 0x ## high ## low ## ull
#define uint64_dconst(high, low)       high ## low ## ull

/* constants as in assigning to a variable */
#define int64_assign_xconst(high, low, dst)   (dst) = (0x ## high ## low ## ll)
#define int64_assign_dconst(high, low, dst)   (dst) = ( high ## low ## ll )

/* the rest all use variables as operands */
#define int64_assign(src,dst)   (dst) = (src)
/* add */
#define int64_add_1op(src,dst)  (dst) = ((dst) + (src))
#define int64_add(s1,s2,dst)    (dst) = ((s1) + (s2))
/* subt */
#define int64_sub_1op(src,dst)  (dst) = ((dst) - (src))
#define int64_sub(s1,s2,dst)    (dst) = ((s1) - (s2))
/* shift */
#define int64_shift_left(src,amt,dst)  (dst) = ((src) << (amt))
#define int64_shift_right(src,amt,dst) (dst) = ((src) >> (amt))
/* mult, div */
#define int64_div_1op(divisor,dst)      (dst) = ((dst) / (divisor))
#define int64_div(dividend,divisor,dst) (dst) = ((dividend) / (divisor))
#define int64_mul_1op(src,dst)          (dst) = ((dst) * (src))
#define int64_mul(src1,src2,dst)        (dst) = ((src1) * (src2))
/* bit ops */
#define int64_bit_and_1op(src,dst)      (dst) = ((dst) & (src))
#define int64_bit_and(s1,s2,dst)        (dst) = ((s1)  & (s2))
#define int64_bit_xor_1op(src,dst)      (dst) = ((dst) ^ (src))
#define int64_bit_xor(s1,s2,dst)        (dst) = ((s1)  ^ (s2))
#define int64_bit_or_1op(src,dst)       (dst) = ((dst) | (src))
#define int64_bit_or(s1,s2,dst)         (dst) = ((s1)  | (s2))
/* tests, return 0 or 1 */
#define int64_lt(s1,s2)                 ((s1) < (s2))
#define int64_gt(s1,s2)                 ((s1) > (s2))
#define int64_leq(s1,s2)                ((s1) <= (s1))
#define int64_geq(s1,s2)                ((s1) >= (s2))
#define int64_eq(s1,s2)                 ((s1) == (s2))
#define int64_is_zero(src)              ((src) == (0ull))
/* miscellaneous */
long uint64_printf(long long value);
long  int64_printf(char *str, unsigned long long value);

#define int96_get_low(src)   (int)(src.low & 0xffffffff)
#define int96_get_mid(src)   (int)( src.low>>32 )
#define int96_get_high(src)  (int)(src.high & 0xffffffff)
#define int96_set_low(src,dst) (dst.low) = ((dst.low) & ((int64)src))
#define int96_set_mid(src,dst) \
    (dst.low) = ( (((int64)src)<<32) | (dst.low &0xffffffff) )
#define int96_set_high(src,dst) (dst.high) = ((int64)(src))

/* conversions between types */
#define int32_to_int64(src,dst)  (dst) = (int64)(src)
#define int32_to_int96(src,dst)  (dst) = (int96)(src)
#define int32_to_int128(src,dst) \
     do{int64_assign_xconst(0,0,dst.high);(dst.low) = (int64)(src);}while(0)
#define int64_to_int32(src,dst)  (dst) = (int)(src & 0xffffffff)
#define int64_to_int96(src,dst)  do{(dst.low) = (src);(dst.high)=0}while(0)
#define int64_to_int128(src,dst) \
     do{int64_assign_xconst(0,0,dst.high);int64_assign(dst.low,src);}while(0)
#define int96_to_int32(src,dst)  (dst) = (int)((src.low) & 0xffffffff)
#define int96_to_int64(src,dst)  (dst) = (src.low)
#define int96_to_int128(src,dst) (dst) = (src)
#define int128_to_int32(src,dst) (dst) = (int)((src.low) & 0xffffffff)
#define int128_to_int64(src,dst) (dst) = (src.low)
#define int128_to_int96(src,dst) (dst) = (src)

/* 128 bit stuff */
/* constants in array init */
#define int128_xconst(h1,l1,h2,l2,dst) \
              { (0x ## h1 ## l1 ## ll), (0x ## h2 ## l2 ## ll ) }
#define int128_dconst(h1,l1,h2,l2,dst) { (h1 ## l1 ## ll), (h2 ## l2 ## ll ) }
#define uint128_xconst(h1,l1,h2,l2,dst) int128_xconst(h1,l1,h2,l2,dst) \
              { (0x ## h1 ## l1 ## ull), (0x ## h2 ## l2 ## ull ) }
#define uint128_dconst(h1,l1,h2,l2,dst) { (h1 ## l1 ## ull), (h2 ## l2 ## ull ) }
/* constants as in assigning to a variable */
/*
#define int64_assign_xconst(h1,l1,h2,l2,dst)                            \
       do{               (dst.low)=(0x ## h1 ## l1 ## ll);              \
			 (dst.high)=(0x ## h2 ## l2 ## ll); }  while(0)

#define int64_assign_dconst(h1,l1,h2,l2,dst)                            \
       do{               (dst.low)=(h1 ## l1 ## ll);                    \
			 (dst.high)=(h2 ## l2 ## ll); }  while(0)
*/

/* the rest all use variables as operands */
#define int128_assign(src,dst)    (dst) = (src)
/* add */
#define int128_add_1op(src,dst)   
#define int128_shift_left()
#define int128_shift_right()
#define int128_add(a,b,d)         (d) = ((a) + (b))
#define int128_div(a,b,d)         (d) = (a) / (b)
#define int128_mul(a,b,d)         (d) = (a) * (b)
#define int128_lt(a,b)            ((a) < (b))
#define int128_gt(a,b)            ((a) > (b))
#define int128_leq(a,b)           ((a) <= (b))
#define int128_geq(a,b)           ((a) >= (b))
#define int128_eq(a,b)            ((a) == (b))
#define int128_bit_and(a,b)       ((a) & (b))
#define int128_bit_xor(a,b)       ((a) ^ (b))
#define int128_bit_or(a,b)        ((a) | (b))
#define int128_printf()

#define int96_assign(a,d)        int128_assign(a,d)
#define int96_shift_left(a,b,d)  (d) = ((a) << (b))
#define int96_shift_right(a,b,d) (d) = ((a) >> (b))
#define int96_add(a,b,d)         (d) = ((a) + (b))
#define int96_div(a,b,d)         (d) = (a) / (b)
#define int96_mul(a,b,d)         (d) = (a) * (b)
#define int96_lt(a,b)            ((a) < (b))
#define int96_gt(a,b)            ((a) > (b))
#define int96_leq(a,b)           ((a) <= (b))
#define int96_geq(a,b)           ((a) >= (b))
#define int96_eq(a,b)            ((a) == (b))
#define int96_bit_and(a,b)       ((a) & (b))
#define int96_bit_xor(a,b)       ((a) ^ (b))
#define int96_bit_or(a,b)        ((a) | (b))
#define int96_printf()

#else

typedef struct {
    int high;
    int  low;
} int64_type;

typedef struct {
    unsigned int high;
    unsigned int  low;
} uint64_type;

#define int64_const(high, low) (0x ## high), (0x  ## low)
#define uint64_const(high, low) (0x ## high), (0x  ## low)

#define int64(high,low)  (high), (low)

#define uint64(high, low) (high), (low)

#define int32_to_int64(a,d) { (d).low = (a); (d).high = ((a) >> 31) * -1; }
#define int32_add_int64(a,d) { (d).low += (a); (d).high = 0; }
 
#define int64_assign(a,d)   { (d).high = (a).high; (d).low = (a).low; }

#define int64_shift_left(a,b,d)  {                                   \
		if((b)==0) {                                         \
		  (d).low = (a).low; (d).high = (a).high;            \
                }		                                     \
		else if((b)>=64) {                                   \
		    (d).low = 0; (d).high = 0;                       \
		}                                                    \
		if((b) < 32) {                                       \
		   (d).high = (a).high << (b) + (a).low >> (32-(b)); \
		   (d).low  = (a).low  << (b);                       \
	        }                                                    \   
                else if( (b) < 64){ /* more than 32 bits */          \
		    (d).high = (d).low << ((b)-32);                  \
		    (d).low = 0;                                     \
		}				                     \
           }

#define int64_shift_right(a,b,d)  {                                  \
		if((b)==0) {                                         \
		    (d).low = (a).low; (d).high = a.high;            \
		}                                                    \  
		else if((b) >= 64) {                                 \ 
		    (d).low = 0; (d).high = 0;                       \
		}                                                    \
		if((b) < 32) {		                             \    
		   (d).high = (a).high >> (b);                       \     
		   (d).low  = (a).low  >> (b) + (a).high << (32-(b));\ 
	        }                                                    \  
	        else if( (b) < 64) { /* more than 32 bits */         \
		    (d).low = (d).high >> (b)-32;                    \
		    (d).high = 0;                                    \
		}                                                    \  
	    }

#define int64_add(a,b,d)  {                                          \ 
/* single expression*/                                               \
		(d).high = (a).high + (b).high +                     \
		   (  ((b).low>>31==1 && (a).low>>31==1) ||          \
		      ( ((b).low>>31==1 || (a).low>>31==1) &&        \
                         (((a).low+(b).low)>>31) ) );                \
	        (d).low = (a).low + (b).low;                         \
	   }

/* multiple expressions
	  unsigned int abit, bbit, dbit, carry;
	  (d).low = (a).low + (b).low;
	  bbit = (b).low >> 31; abit = (a).low >> 31;
	  dbit = (d).low >> 31;
	  carry = 0;
	  if(bbit == 1 && abit == 1) carry = 1;
	  if( (bbit == 1 || abit==1) && dbit == 1) carry = 1;
	  (d).high = (a).high + (b).high + carry;
*/

/* 16 bit adds 
  t1 = ( (a).low & 0xffff + (b).low & 0xffff )
  t2 = (a).low >> 16 + (b).low >> 16 + (t1 && 0xffff0000);
  t3 = ( (a).high & 0xffff + (b).high & 0xffff ) + (t2 & 0xffff0000);
  t4 = ( (a).high >> 16 + (b).high >> 16 + (t3 && 0xffff0000);
*/

#define int64_div(a,b,d)
            

#define int64_mul(a,b,d)             


#define int64_lt(a,b) (   ((a).high < (b).high) ||      \
    ( ((a).high == (b).high) && ( ((unsigned)(a).low) < ((unsigned)(b).low))))

                        
#define int64_gt(a,b) (   ((a).high >  (b).high) ||      \
    ( ((a).high == (b).high) && ( ((unsigned)(a).low) > ((unsigned)(b).low) )))

#define int64_lt(a,b) (   ((a).high < (b).high) ||      \
    ( ((a).high == (b).high) && ( ((unsigned)(a).low) < ((unsigned)(b).low) )))

#define int64_leq(a,b) (   ((a).high <= (b).high) ||    \
   ( ((a).high == (b).high) && ( ((unsigned)(a).low) <= ((unsigned)(b).low) )))

#define int64_geq(a,b) (   ((a).high >= (b).high) ||    \
   ( ((a).high == (b).high) && ( ((unsigned)(a).low) >= ((unsigned)(b).low) )))

#define int64_eq (((a).high == (b).high) && ((a).low == (b).low))

#define int64_bit_and(a,b,d) {                      \
		  (d).high = (a).high & (b).high;   \
		  (d).low = (a).low & (b).low;      \
	      }

#define int64_bit_xor(a,b,d) {                      \
		  (d).high = (a).high ^ (b).high;   \
		  (d).low = (a).low & (b).low;      \
	      }

#define int64_bit_or(a,b,d) {                       \
		  (d).high = (a).high ^ (b).high;   \
		  (d).low = (a).low ^ (b).low;      \
	      }


#endif
