/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/tadti/builtin.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef tadti_builtin
#define tadti_builtin

/* 
* Routine: tadti_intersect
*  
* Description: Returns the indeterminate overlap of two intervals, if
*	       they intersect.  Otherwise leave the result
*              unchanged.
*
* Arguments:  gamma -- (IN): plausibility 
*             i1, i2 -- (IN): intervals for intersection
*             result -- (OUT): result interval if intervals intersect 
*
* Return Value: Error code
* 
* Errors: tadt_invalid_interval
*
* Side Effects: Assignment to passed result structure
*/

tadt_error_type tadti_intersect(int gamma, interval_type i1, interval_type i2, interval_type result);

/* 
*  Routine: tadti_first
*  
*  Description: Returns the earlier of two events using plausibility
*
* Arguments:  gamma -- (IN): plausibility 
*             event1, event2 -- (IN): two input events
*             result -- (OUT) earliest event.
*
*  Return Value: Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadti_first(int gamma, event_type event1, event_type event2, event_type result);

/* 
*  Routine: tadti_last
*  
*  Description: Returns the later event of two events using plausibility
*
* Arguments:  gamma -- (IN): plausibility 
*             event1, event2 -- (IN): two input events
*             result -- (OUT) earliest event.
*
*  Return Value:  Error code
* 
*  Errors: None
*
*  Side Effects: Assignment to passed result structure
*/

tadt_error_type tadti_last(int gamma, event_type event1, event_type event2, event_type result);

void tadti_shrink_s(int amount, event_type event, event_type result);

void tadti_shrink_t(int amount, event_type event, event_type result);

bool tadti_adjust(int gamma, interval_type interval, interval_type result);


#endif
