/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/ucs/calendric.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef ucs_calendric
#define ucs_calendric
/* calendric.h
*
*  This module implements UCS services for calendric
*  system activation.  Routines in this module are called
*  by the query processing system at compile-time for 
*  calendric system binding, and at run-time for calendric
*  system activation.
*
*/

/*  
*  Routine:  ucs_is_calendric_system
*
*  Description:  Determines if the parameter is the name of a calendric system
*
*  Arguments: calendric_system_name -- (IN) : the name of the calendric system
*
*  Return Value: bool : TRUE if the calendric system is defined
*                       FALSE otherwise
*
*  Errors: None  
*  
*  Side Effects: None
*/
bool ucs_is_calendric_system(char *calendric_system_name);

/*  
*  Routine:  ucs_declare_global_calendric_system
*
*  Description:  Inform uniform calendric support that a 
*		 new global calendric system has been selected
*
*  Arguments: calendric_system_name -- (IN) : the name of the calendric system
*
*  Return Value: Error code
*
*  Errors: None, assumes that ucs_is_calendric_system was called previously
*    
*  Side Effects: Changes current calendric system in UCS
*/
error_type ucs_declare_global_calendric_system(char *calendric_system_name);

/*  
*  Routine:  ucs_declare_local_calendric_system
*
*  Description:  Inform uniform calendric support that a 
*		 new local calendric system has been selected
*
*  Arguments: calendric_system_name -- (IN) : the name of the calendric system
*
*  Return Value: Error code
*
*  Errors: None, assumes that ucs_is_calendric_system was called previously
*    
*  Side Effects: Changes current calendric system in UCS
*/
error_type ucs_declare_local_calendric_system(char *calendric_system_name);
#endif
