/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/newh/ucs/input.h
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#ifndef ucs_input
#define ucs_input
/* input.h
*
*  This module implements UCS operations for temporal
*  constant to time-stamp conversion.
*
*/

/*  
*  Routine:  ucs_e_string_to_e
*
*  Description:  Convert an event string to a timestamp according to the
*                specification of the Event Input Format,  Locale, 
*                Override Input Epoch, Beginning, and Forever properties
*
*  Arguments: string --     (IN) :  the event in calendar specific form 
*             event  --     (OUT):  timestamp for the event
*
*  Return Value: Error code
*
*  Errors: ucs_string_invalid
*  
*  Side Effects: Overwrites event
*/
ucs_error_type ucs_e_string_to_e(char *string, event_type event);

/*  
*  Routine:  ucs_i_string_to_i
*
*  Description:  Convert an interval string to a timestamp according to the
*                specification of the Interval Input Format, Locale,  and
*                Override Input Epoch properties 
*
*  Arguments: string      -- (IN) :  the interval in calendar specific form 
*             interval    -- (OUT):  timestamp for the interval
*
*  Return Value: Error code
*
*  Errors: ucs_string_invalid
*  
*  Side Effects: Overwrites interval
*/
ucs_error_type ucs_i_string_to_i(char *string, interval_type interval);

/*  
*  Routine:  ucs_s_string_to_s
*
*  Description:  Convert a span string to a timestamp according to the
*                specification of the Span Input Format and Override
*                Input Epoch properties
*
*  Arguments: string   --  (IN) :  the span in calendar specific form 
*             span     --  (OUT):  timestamp for the span 
*
*  Return Value: Error code
*
*  Errors: ucs_string_invalid
*  
*  Side Effects: Overwrites span
*/
ucs_error_type ucs_s_string_to_s(char *string, span_type span);
#endif
