/***********************************************************************\ 
*                                                                       * 
*   File: calendric_system/tadt/tadt_memory.c
*                                                                       * 
*   The MultiCal System is free software in the public domain; you can  * 
*   redistribute it and/or modify it as you wish. We ask that you       * 
*   retain credits referencing the University of Arizona and that you   * 
*   identify any changes you make.                                      * 
*                                                                       * 
*   Report problems to multical@cs.arizona.edu                          * 
*   Direct all inquiries to:    The MultiCal Project                    * 
*                               Department of Computer Science          * 
*                               University of Arizona                   * 
*                               Tucson, AZ 85721                        * 
*                               U.S.A.                                  * 
*                                                                       * 
\***********************************************************************/ 
#include <stdlib.h>
#include "system_wide.h"
#include "tadt.h"
#include "math.h"

/* 
 * tadt global variables follow
 */


/* Tadt internal helper code to quickly allocate events */
#define INTERNAL_MAX 50
static event_space_type working_events[INTERNAL_MAX];
static int current_working_event;
void reset_quick_space_allocation() {current_working_event=0;}
event_type quickly_allocate_event() {return (event_type)&(working_events[current_working_event++]);}

dist_space_type uniform;
dist_type distributions[MAX_DISTRIBUTION] = {&uniform};
seconds_type tadt_secs1;
event_type   tadt_event1;
span_type    tadt_span1;
span_type    tadt_zero_span;
unsigned long leap_second_table[MAX_LEAP_SECOND] = {0,1,2,3,4,5,6,7,8,9,10,
						   11,12,13,14,15,16,17,18,19,
						   20,21,22,23,24,25,26};
unsigned long leap_second_probe_table[MAX_LEAP_SECOND] = {0,
                           78796800,
                           78796800,
                           78796800,
                           78796800,
                           78796800,
                           78796800,
                           78796800,
                           78796800,
                           78796800,
                           78796800,
                           94694400,
                          126230400,
                          157766400,
                          189302400,
                          220924800,
                          252460800,
                          283996800,
                          315532800,
                          362793600,
                          394329600,
                          425865600,
                          489024000,
                          536457600,
                          599616000,
                          631152000,
                          678412800};


resolution_type combined_flags[MAX_RESOLUTIONS_OF_EVENTS][MAX_RESOLUTIONS_OF_EVENTS];
size_type sizes[MAX_RESOLUTIONS_OF_EVENTS][MAX_PRECISION];

#ifdef INDETERMINATE
int32 chunk32_table[HOW_MANY_CHUNKS][MAX_CHUNK];
int64 chunk64_table[HOW_MANY_CHUNKS][MAX_CHUNK];
int96 chunk96_table[HOW_MANY_CHUNKS][MAX_CHUNK];
dist_space_type          dist_table[HOW_MANY_DISTS];
#endif

#ifdef INDETERMINATE
unsigned char mult_table[MULT_TABLE_MAX][MULT_TABLE_MAX];
unsigned int uniform_tree_height = 0;
int pregammas[101];
int maxqueue = 0;
tree_node_type spare_nodes[10000];
int allocated = 0;
tree_node_type *uniform_tree;
queue_node_type queue[PRECISION*8 + 2];

tree_node_type *build_spike_tree(unsigned int height) {
  tree_node_type *node, *build_uniform_tree();

  node = &spare_nodes[allocated++];
  
  if (height < 8)  {
    node->value = 0;
    node->left = 0 /*NULL*/;
    node->leftrods = 0;
    node->rightrods = PRECISION;
    node->right = build_spike_tree(height + 1);
    }
  else {
    node->value = 0;
    node->left = 0 /*NULL*/;
    node->right = build_uniform_tree(PRECISION,0,0,PRECISION);
    }
  return node;
}

tree_node_type *build_uniform_tree(unsigned int current, unsigned int height,
				   unsigned int lower, unsigned int upper) {
  tree_node_type *node;

  node = &spare_nodes[allocated++];
  node->value = lower + ((upper - lower) >> 1);
  if (current == 1) {
    node->leftrods = node->value;
    node->rightrods = PRECISION - node->value - 1;
    /*
    if (upper == PRECISION) node->value = PRECISION;
    if (upper == PRECISION) node->leftrods = PRECISION;
    if (upper == PRECISION) node->rightrods = 0;
    */
#ifdef TESTING
  printf("node %d leftrods %d rightrods %d\n", node->value, node->leftrods, node->rightrods);
#endif
    node->left = 0 /*NULL*/;
    node->right = 0 /*NULL*/;
    }
  else {
    node->leftrods = node->value;
    node->rightrods = PRECISION - node->value;
#ifdef TESTING
  printf("node %d leftrods %d rightrods %d\n", node->value, node->leftrods, node->rightrods);
#endif
    node->left = (void *)build_uniform_tree(current >> 1,height+1,lower,node->value);
    node->right = (void *)build_uniform_tree(current >> 1,height+1,node->value,upper);
    }
  if (height > uniform_tree_height) uniform_tree_height = height;
  /*
  node->height = height;
  */
  return node;
}
#endif

/*  
*  Routine: tadt_init
*
*  Description: Allocate globals shared in the tadt system.
*               These globals are shared variables such as seconds structures.
*
*  Arguments:   None
*
*  Return Value: None
*
*  Errors: None
*  
*  Side Effects: overwrites the tadt global pointers
*/
void tadt_init()
{
  event_space_type temp_present;
  int i, j;
  int32 temp32, chunk32;
  int64 temp64, chunk64;
  int96 temp96, chunk96;

#ifdef INDETERMINATE
  double f;
  int temp1;
#endif 

  tadt_present_as_of_query(TRUE, &temp_present);

  tadt_secs1 = tadt_allocate_seconds(packed_determinate);
  tadt_event1 = tadt_allocate_e(packed_determinate);
  tadt_span1 = tadt_allocate_s(packed_determinate);
  tadt_zero_span = tadt_allocate_s(upack);
  tadt_zero_span->upack96.granules = 0; /* Should cover 32, 64, and 96 */

  /* Reset the quickly accessible event allocation stuff */
  reset_quick_space_allocation();

  /* Build the sizes Table */
  for (i=0; i < MAX_RESOLUTIONS_OF_EVENTS; i++) {
    for (j=0; j < MAX_PRECISION; j++) {
      sizes[i][j] = upack32;
      }
    }

  sizes[packed_determinate][9] = upack64;
  sizes[packed_determinate][10] = upack64;
  sizes[packed_determinate][11] = upack64;
  sizes[packed_determinate][12] = upack64;
  sizes[packed_determinate][13] = upack64;
  sizes[packed_determinate][14] = upack64;
  sizes[packed_determinate][15] = upack64;
  sizes[packed_determinate][16] = upack64;
  sizes[packed_determinate][17] = upack64;
  sizes[packed_determinate][18] = upack64;

  sizes[packed_determinate][19] = upack96;
  sizes[packed_determinate][20] = upack96;
  sizes[packed_determinate][21] = upack96;
  sizes[packed_determinate][22] = upack96;
  sizes[packed_determinate][23] = upack96;
  sizes[packed_determinate][24] = upack96;
  sizes[packed_determinate][25] = upack96;
  sizes[packed_determinate][26] = upack96;
  sizes[packed_determinate][27] = upack96;

  for (i=0; i < MAX_RESOLUTIONS_OF_EVENTS; i++) 
    for (j=0; j < MAX_RESOLUTIONS_OF_EVENTS; j++) 
      combined_flags[i][j] = MAX_RESOLUTIONS_OF_EVENTS;

  combined_flags[upack32][upack32] = upack32upack32;
  combined_flags[upack32][upack64] = upack32upack64;
  combined_flags[upack32][upack96] = upack32upack96;
  combined_flags[upack32][upackind] = upack32upackind;
  combined_flags[upack32][special] = upack32special;
  combined_flags[upack64][upack32] = upack64upack32;
  combined_flags[upack64][upack64] = upack64upack64;
  combined_flags[upack64][upack96] = upack64upack96;
  combined_flags[upack64][upackind] = upack64upackind;
  combined_flags[upack64][special] = upack64special;
  combined_flags[upack96][upack32] = upack96upack32;
  combined_flags[upack96][upack64] = upack96upack64;
  combined_flags[upack96][upack96] = upack96upack96;
  combined_flags[upack96][upackind] = upack96upackind;
  combined_flags[upack96][special] = upack96special;
  combined_flags[special][upack32] = specialupack32;
  combined_flags[special][upack64] = specialupack64;
  combined_flags[special][upack96] = specialupack96;
  combined_flags[special][upackind] = specialupackind;
  combined_flags[special][special] = specialspecial;
  combined_flags[upackind][upack32] = upackindupack32;
  combined_flags[upackind][upack64] = upackindupack64;
  combined_flags[upackind][upack96] = upackindupack96;
  combined_flags[upackind][upackind] = upackindupackind;
  combined_flags[upackind][special] = upackindspecial;

  combined_flags[indistinguishable][upack32] = indistinguishableupack32;
  combined_flags[indistinguishable][upack64] = indistinguishableupack64;
  combined_flags[indistinguishable][upack96] = indistinguishableupack96;
  combined_flags[indistinguishable][upackind] = indistinguishableupackind;
  combined_flags[indistinguishable][special] = indistinguishablespecial;
  combined_flags[indistinguishable][incomparable] = indistinguishableincomparable;

  combined_flags[upack32][indistinguishable] = upack32indistinguishable;
  combined_flags[upack64][indistinguishable] = upack64indistinguishable;
  combined_flags[upack96][indistinguishable] = upack96indistinguishable;
  combined_flags[upackind][indistinguishable] = upackindindistinguishable;
  combined_flags[special][indistinguishable] = specialindistinguishable;
  combined_flags[incomparable][indistinguishable] = incomparableindistinguishable;

  combined_flags[indistinguishable][indistinguishable] = indistinguishableindistinguishable;

  combined_flags[incomparable][upack32] = incomparableupack32;
  combined_flags[incomparable][upack64] = incomparableupack64;
  combined_flags[incomparable][upack96] = incomparableupack96;
  combined_flags[incomparable][upackind] = incomparableupackind;
  combined_flags[incomparable][special] = incomparablespecial;

  combined_flags[upack32][incomparable] = upack32incomparable;
  combined_flags[upack64][incomparable] = upack64incomparable;
  combined_flags[upack96][incomparable] = upack96incomparable;
  combined_flags[upackind][incomparable] = upackindincomparable;
  combined_flags[special][incomparable] = specialincomparable;

  combined_flags[incomparable][incomparable] = incomparableincomparable;

#ifdef INDETERMINATE

  for (i=0;i<MULT_TABLE_MAX;i++) for (j=0;j<MULT_TABLE_MAX;j++) mult_table[i][j] = i*j;

#define GAMMA_MULT PRECISION * PRECISION / 100.0
  for (i=0;i<101;i++) {
    f = i;
    pregammas[i] = (int) ceil(f * GAMMA_MULT);
    /* printf("%d %d %f \n", i, pregammas[i], (i * (GAMMA_MULT)));*/
    }

  uniform_tree = build_uniform_tree(PRECISION,0,0,PRECISION);
  /*
  uniform_tree = build_spike_tree(1);
  */


  /* Build the 32 bit Chunk Table */
  /*
  LOAD32(86400,chunk32);
  */
  LOAD32(24,chunk32);
  for (i=0; i < HOW_MANY_CHUNKS; i++) {
    for (j=0; j < MAX_CHUNK; j++) {
      /*
      chunk32_table[i][j].upack.ts_type = upack32;
      */
      LOAD32(i,temp32);
      /*
      MULT32(temp32,chunk32,chunk32_table[i][j].upack32.granules);
      */
      MULT32(temp32,chunk32,chunk32_table[i][j]);
      }
      LOAD32(1,temp32);
      MULT32(chunk32,temp32,chunk32);
    }

  /* Build the 64 bit Chunk Table */
  LOAD64(1,chunk64);
  for (i=0; i < HOW_MANY_CHUNKS; i++) {
    for (j=0; j < MAX_CHUNK; j++) {
      /*
      chunk64_table[i][j].upack.ts_type = upack64;
      */
      LOAD64(i,temp64);
      /*
      MULT64(temp64,chunk64,chunk64_table[i][j].upack64.granules);
      */
      MULT64(temp64,chunk64,chunk64_table[i][j]);
      }
    LOAD64(1,temp64);
    MULT64(chunk64,temp64,chunk64);
    }

  /* Build the 96 bit Chunk Table */
  LOAD96(1,chunk96);
  for (i=0; i < HOW_MANY_CHUNKS; i++) {
    for (j=0; j < MAX_CHUNK; j++) {
      /*
      chunk96_table[i][j].upack.ts_type = upack96;
      */
      LOAD96(i,temp96);
      /*
      MULT96(temp96,chunk96,chunk96_table[i][j].upack96.granules);
      */
      MULT96(temp96,chunk96,chunk96_table[i][j]);
      }
    LOAD96(1,temp96);
    MULT96(chunk96,temp96,chunk96);
    }

  /* Build the Distribution Table */
  temp1 = 1;
  for (i=SIG_BITS; i>0; i--) temp1 = temp1*2;

  for (i=DIST_MIN; i<=DIST_MAX; i++) 
       dist_table[UNIFORM].floor[i] = (temp1*(i+1))/(DIST_MAX + 1);
  for (i=DIST_MIN; i<=DIST_MAX; i++) 
       dist_table[UNIFORM].ceiling[DIST_MAX - i] = (temp1*(DIST_MAX - i))/(DIST_MAX + 1);
  dist_table[UNIFORM].floor[DIST_MAX] = temp1 - 1;

  for (i=DIST_MIN; i<=DIST_MAX; i++) {
    uniform.ceiling[i] = (temp1*(i))/DIST_MAX;
    if ((temp1*(i)) % DIST_MAX) uniform.ceiling[i]++;
    uniform.floor[i] = (temp1*(i))/DIST_MAX;
    }

#endif

}  

/*  
*  Routine: tadt_malloc
*
*  Description: Allocate storage from the TADT memory heap.  Marks a region
*               of mem_size bytes which is now in use.  If no more memory
*               can be allocated, the behaviour of this function is undefined.
*               Currently, this function zero's out the allocated space, 
*               but this behavior is subject to change.
*
*  Arguments: size -- (IN): the number of bytes to allocate
*
*  Return Value: The address of allocated memory
*
*  Errors: None
*  
*  Side Effects: Some of the TADT heap is marked as in use. This may lead
*                to memory leaks later on.
*/

void *tadt_malloc(int size)
{
  int i;
  char *where, what, *asdf;
  

  what = 0;
  where = (char *)malloc(size);
  asdf = where;
  for (i = 0; i < size; i++) *where++ = what;

  return (void *)asdf;
}

/*  
*  Routine:  tadt_free
*
*  Description: Frees previously allocated storage
*
*  Arguments: address -- (IN):  address of previously allocated storage
*
*  Return Value: TRUE if the address was de-allocated, FALSE otherwise
*
*  Errors: None
*
*  Side Effects: If the address has not been allocated previously, then
*                this will could lead to internal errors.
*/

bool tadt_free(void *address)
{
#ifdef stupid_free
  return (bool)free(address);
#else
  free(address);
  return TRUE;
#endif
}

/* 
*  Routine: tadt_allocate_e
*
*  Description: Memory allocation for an event
*
*  Arguments: resolution -- (IN): which resolution to allocate
*
*  Return Value: Address of allocated event
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

event_type tadt_allocate_e(resolution_type resolution)
{
  event_type event;

  /* Always allocate the biggest */
  event = (event_type)tadt_malloc(sizeof(event_space_type));
  event->generic.ts_type = resolution;

  return event;
}

/* 
*  Routine: tadt_allocate_s
*  
*  Description: Memory allocation for an span
*
*  Arguments: resolution -- (IN): which resolution to allocate
*
*  Return Value: Address of allocated span
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

span_type tadt_allocate_s(resolution_type resolution)
{
  span_type span;

  /* Always allocate the biggest */
  span = (span_type)tadt_malloc(sizeof(span_space_type));
  span->generic.ts_type = resolution;

  return span;
}

/* 
*  Routine: tadt_allocate_i
*  
*  Description: Memory allocation for an interval
*
*  Arguments: resolution1 -- (IN): resolution of beginning event
*                                  resolution of the ending event
*
*  Return Value: Address of allocated interval
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

interval_type tadt_allocate_i(resolution_type resolution1, resolution_type resolution2)
{
  interval_type interval;

  /* Always allocate the biggest */
  interval = (interval_type)tadt_malloc(sizeof(interval_space_type));
  interval->start = tadt_allocate_e(resolution1);
  interval->terminate = tadt_allocate_e(resolution2);
  return interval;
}

/* 
*  Routine: tadt_allocate_ts   
*  
*  Description: Memory allocation for an time-stamp.  Allocates the max space.
*
*  Arguments: None
*
*  Return Value: Address of allocated time-stamp
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

timestamp_type tadt_allocate_ts()
{
  return (timestamp_type)tadt_malloc(sizeof(timestamp_space_type));
}

/* 
*  Routine: tadt_allocate_vs_struct
*  
*  Description: Memory allocation for an variable span structure.
*
*  Arguments: None
*
*  Return Value: Address of allocated vs_struct
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

vs_type tadt_allocate_vs_struct()
{
  return (vs_type)tadt_malloc(sizeof(vs_space_type));
}

/* 
*  Routine: tadt_allocate_avg_state_e
*  
*  Description: Memory allocation for an average event state structure.
*
*  Arguments: None
*
*  Return Value: Address of allocated average event state structure
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

avg_state_e tadt_allocate_avg_state_e()
{
  avg_state_e size;

  return (avg_state_e)tadt_malloc(sizeof(*size));
}


/* 
*  Routine: tadt_allocate_avg_state_fs
*  
*  Description: Memory allocation for an average fixed span state structure.
*
*  Arguments: None
*
*  Return Value: Address of allocated average fixed span state structure
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

avg_state_fs tadt_allocate_avg_state_fs()
{
  avg_state_fs size;
  return (avg_state_fs)tadt_malloc(sizeof(*size));
}

/* 
*  Routine: tadt_allocate_string_struct
*  
*  Description: Memory allocation for string structure
*
*  Arguments: length -- (IN): the size of the string to allocate
*
*  Return Value: Address of allocated string structure
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

string_struct tadt_allocate_string_struct(int capacity)
{
  string_struct temp;

  temp = (string_struct)tadt_malloc(sizeof(*temp));
  temp->capacity = capacity;
  temp->length = 0;
  temp->string = tadt_malloc(capacity+1);

  return temp;
}

/* 
*  Routine: tadt_allocate_seconds
*  
*  Description: Memory allocation for seconds structure
*
*  Arguments: resolution -- (IN): Resolution of allocated thing
*
*  Return Value: Address of allocated seconds structure
* 
*  Errors: None
*
*  Side Effects: Mallocs space
*/

seconds_type tadt_allocate_seconds(resolution_type resolution)
{
  seconds_type sec;

  sec = (seconds_type)tadt_malloc(sizeof(seconds_space_type));
  sec->resolution = resolution;
  return sec;
}
